// ****************** MUST INCLUDE LINES BELOW*****************************
#include <lib6302.h>
#define MYSERIAL Serial
lib6302 my6302(234375/8,12,16,&Serial);
// Set up the analog-to-digital library, with a buffersize
#define sampleBufN 500
LIB6302_ADC(sampleBufN+100);  // ADC sample buffer of 100+sampleBufN
// ****************** MUST INCLUDE LINES ABOVE*****************************

// Constants for Users to Modify 
// ************************************************************

// Calibration variables
#define UseMatlabGenFile false // Set to true AFTER calibration

// motor, theta (angle), and backEMF calibration.
#define thetaOffset 0.15 //  theta_a = zero when arm is horizontal
#define nomCmdL 0.535      // Motor cmd for nominal speed.
#define nomCmdR 0.465     // Motor cmd for nominal speed.
#define nomBemf 0.16    // Nominal Rotations per second.
#define toggleAngle 0.0 // Toggle angle >0 for step, <0 for sine
#define toggleFreq 0.25  // Frequency of toggling
#define plotIncrement 10 // 1: Plot every sample, 10: Plot every 10th sample

// End of Constants and code for Users to Modify 
// SEE END OF FILE FOR USER-ALTERABLE CONTROL LOOP!!!
// ************************************************************************
#define filtAlpha 0.1
#define controllerUpdatePeriod 1000 // Update period in microseconds.
#define mBack 5
#define TWOPI 6.2831853

// Typedef for the three and four long vectors, and 3x3 and 4x4 matrices
typedef float vec3[3];
typedef vec3 mat3x3[3];
typedef float vec4[4];
typedef vec4 mat4x4[4];

// Fill in Ad,Bd,Cd,Ld,Kd, by copy and past if obs6302.h is empty
float Kr = 0.1; // y_d scale factor.
vec3 K = { 0.1, 0.01, 0.0 }; // Vector of Control Feedback Gains
float Ksum = 0.0;  // For integral term.

vec3 Ld = { 0.0,0.0,0.0 }; // Observer gains.

vec3 Bd = { 0,0,0 }; // Prop model.
vec3 Cd = { 0.0,   0,   0 };
mat3x3 Ad = {
  {0.0, 0.0, 0.0},
  {0.0, 0.0, 0.0},
  {0.0, 0.0, 0.0}
};

// Estimated and Measured state vectors
#define stateBound 500.0
vec3 xPred = { 0.0, 0.0, 0.0 };
float xSum = 0.0;

//*************Matrix operations borrowed from linmath.h
static inline void mat3x3_mul_vec3(vec3 r, mat3x3 M, vec3 v)
{
  int i, j;
  for(j=0; j<3; ++j) {
    r[j] = 0.0;
    for(i=0; i<3; ++i)
      r[j] += M[j][i] * v[i]; // Edited to untranspose M
  }
}

static inline void copy_bound_vec3(vec3 r, vec3 a, float bound)
{
  int j;
  for(j=0; j<3; ++j) {
    if(a[j] > bound) r[j] = bound;
    else if(a[j] < -bound) r[j] = -bound;
    else r[j] = a[j];
  }
}

#define LINMATH_DEFINE_VEC(n) \
static inline void vec##n##_add(vec##n r, vec##n const a, vec##n const b) \
{ \
  int i; \
  for(i=0; i<n; ++i) \
    r[i] = a[i] + b[i]; \
} \
static inline void vec##n##_sub(vec##n r, vec##n const a, vec##n const b) \
{ \
  int i; \
  for(i=0; i<n; ++i) \
    r[i] = a[i] - b[i]; \
} \
static inline void vec##n##_scale(vec##n r, vec##n const v, float const s) \
{ \
  int i; \
  for(i=0; i<n; ++i) \
    r[i] = v[i] * s; \
} \
static inline float vec##n##_mul_inner(vec##n const a, vec##n const b) \
{ \
  float p = 0.; \
  int i; \
  for(i=0; i<n; ++i) \
    p += b[i]*a[i]; \
  return p; \
} 
LINMATH_DEFINE_VEC(3)
LINMATH_DEFINE_VEC(4)
//************END BORROWED FROM LINMATH.H


// SETTING UP THE MONITOR DISPLAY (SLIDERS AND PLOTS)
// ************************************************************************
String sliders = "S~Pause~-1.0~1.0~S~Theta_d(Neg to Toggle)~-2.5~2.5~S~Toggle Freq(Hz)~0.2~4~S~Meas(0)->Est(1)~0~1~";
#define SLIDER_PAUSE 0
#define SLIDER_TD 1
#define SLIDER_HZ 2
#define SLIDER_EST 3


String yMeasEstDes = "P~yMeas yPred yDes~-5~5~500~3~";
String measEst1 = "P~Omega OmegaEst~-5~5~500~2~";
String measEst2 = "P~VBemf VbemfEst~-5~5~500~2~";
String cmd = "P~DeltaMotorCmd~-5~5~500~1~";
String Integ = "P~Integral~-20~20~500~1~";

// Now assemble the various configuration strings.
String config_message = sliders+yMeasEstDes+measEst1+measEst2+cmd;//+Integ;
//String config_message = sliders+measEst+measEst+measEst+yMeasEstDes;
#define PLOT_YM 0
#define PLOT_YE 1
#define PLOT_YD 2
//#define PLOT_M0 3
//#define PLOT_E0 4
#define PLOT_M1 3
#define PLOT_E1 4
#define PLOT_M2 5
#define PLOT_E2 6
#define PLOT_DM 7
//#define PLOT_INT 9

// ************************************************************************
//  Done with Monitor Display Setup*********************

// Calculate the backEMF = V_motor - R_motor*I_motor.
// Uses generic analog read function to avoid interrupt conflicts between 
// tach and ADC.
#define offsetMotorV  0.5 
#define scaleMotorV 2.0   // vMotor 
#define scaleMotorC 0.66  // iMotor = (V(iSense)/6)/0.25ohms = 0.66
#define Rm 1.1

float getParallelBackEMF(int pinVA, int pinVB, int pinCA, int pinCB, int nAvg) {
  // Read motor current and motor voltage.
  float sumBemf = 0.0;
  float absMotorC = 0.0;
  for(int i = 0; i < nAvg; i++) {
    float absMotorCA = scaleMotorC*my6302.adcGetAvg(pinCA); // Sensor reads abs(crnt).
    float absMotorCB = scaleMotorC*my6302.adcGetAvg(pinCB); // Sensor reads abs(crnt).
    float motorVA = scaleMotorV*(my6302.adcGetAvg(pinVA) - offsetMotorV);
    float motorVB = scaleMotorV*(my6302.adcGetAvg(pinVB) - offsetMotorV);
    absMotorC += 0.5*(absMotorCA + absMotorCB);
    float motorV = 0.5*(motorVA+motorVB);
    
    // BackEMF is motorV minus Rmotor*absMotorC, but signs flip if motorV < 0.
    float backEMF = motorV;
    float RmDrop = 0.5*(absMotorCA+absMotorCB)*Rm;
    backEMF -= ((backEMF >  0.0) ? RmDrop : -RmDrop); 
    sumBemf += abs(backEMF);
  }
  return(sumBemf/float(nAvg));
}


// Control loop setup
// ************************************************************************

// Arm lab pin definitions
float deltaTSecs = 0.0;
float mDt = 0.0;
float headRoom = float(controllerUpdatePeriod);
float pastTheta[mBack+1];  // Stores past errors for deriv approx
float SumMax = 0.0;
float uMax = 0.0;

void setup() {
  MYSERIAL.begin(1000000);
  my6302.setup(config_message, 1.0);
  my6302.startPWM(true); // start parallel drive pwm, A+B in sync.
  my6302.setupADC(16,true);
  
#if UseMatlabGenFile
#include "obs6302.h"
#endif

  deltaTSecs = float(controllerUpdatePeriod)*1.0e-6;
  mDt = float(mBack)*deltaTSecs;
  headRoom = float(controllerUpdatePeriod);
  
  SumMax = 50.0;
  for (int i = mBack; i >= 0; i--) pastTheta[i] = 0.0;
  float nomCmdMin = min(nomCmdL,nomCmdR);
  float nomCmdMax = max(nomCmdL,nomCmdR);
  uMax = min(nomCmdMin, 1-nomCmdMax);

}

// return val bounded by [-Thresh/Scale, Thresh/Scale]
double boundBiD(double val,double Thresh, double Scale)
{
  if(Scale > 0.01) Thresh *= 1.0/Scale;
  
  if(val > Thresh) val = Thresh;
  if(val < -Thresh) val = -Thresh;
  return(val);
}

float oldTheta_d = 0.0;
int loopCntr = 0;
float updateThetaD(float tD, float FreqD) {
    // Determine the fraction of a period = 1/desiredFreq.
    float periodFrac = float(loopCntr)*deltaTSecs*FreqD;
    if(periodFrac > 1.0) loopCntr = 0;
    else loopCntr  += 1;
    
    // Evaluate sinusoid, or flip sign for steps, with filter.
    if (tD < 0) tD *= -sin(TWOPI*periodFrac); 
    else {
      if (periodFrac > 0.5) tD *= -1.0;
      tD = (1-filtAlpha)*oldTheta_d + filtAlpha*tD;
      oldTheta_d = tD;
    }
    return(tD);
}

//********************** Main USER ALTERABLE Control Loop **********************
//**************************************************************************


elapsedMicros loopTimer = 0;
float xsum = 0.0;
int loopCounter = 0;
uint16_t avgs = 1;

void loop() {  
  
  // Initializes, gets GUI updates.
  my6302.startloop();

  // Get parameters
  float pauseVal= my6302.getSlider(SLIDER_PAUSE);
  float theta_d_in = toggleAngle + my6302.getSlider(SLIDER_TD);
  float desiredFreq = toggleFreq + my6302.getSlider(SLIDER_HZ); 
  float estimateParam =  my6302.getSlider(SLIDER_EST); 

   // Headroom is the faction of the period still remaining after the loop completes.
  float headRoomThis = float(controllerUpdatePeriod - int(loopTimer));
  headRoom = min(headRoomThis,headRoom);
  
  // Start exactly one period since last start.
  while (int(loopTimer) < controllerUpdatePeriod) {};
  loopTimer = 0;

  // Get slider desired value and possibly toggle.
  float theta_d = updateThetaD(theta_d_in,desiredFreq);

  //***** Read Sensors -> compute state -> u=Kr*yd-K*X -> output
  //************************************************************
  
  // Read angle sensor
  float theta_a = TWOPI*(0.5 - my6302.adcGetAvg(A1)) + thetaOffset; 
   
  // Update theta history and compute omega.
  for (int i = mBack; i > 0; i--) pastTheta[i] = pastTheta[i-1];
  pastTheta[0] = theta_a;
  float omega_a = (theta_a - pastTheta[mBack])/mDt;

  // Compute deltaBemf.
  float bEmf = getParallelBackEMF(VsenseB,VsenseA,IsenseB,IsenseA,2);
  float deltaBemf = abs(bEmf) - nomBemf;

  // Get theta, dTheta/dt, deltaBemf
  vec3 xMeas;
  xMeas[0] = theta_a;
  xMeas[1] = omega_a;
  xMeas[2] = deltaBemf;

  // Compute yMeas = theta_a, yPred=Cd*xPred, y_d.
  // theta_a = yMeas = vec3_mul_inner(Cd,xMeas);
  float yMeas = theta_a;
  float y_d = theta_d;

  // Update the integral of the difference yMeas-y_d.
  xSum = boundBiD(xSum+deltaTSecs*(yMeas-y_d), stateBound, Ksum);

  // Compute u = Kr*y_d - (K*X)
  float u = Kr*y_d - Ksum*xSum;  // Update the non-estimated part of u.
  if(my6302.getSlider(SLIDER_EST) > 0.0) u -= vec3_mul_inner(K,xPred);  // Use predicted state
  else u -= vec3_mul_inner(K,xMeas);  // Use measured state

  // State Update: xPred = Ad*xEst + Bd*u + Ld*(yMeas - yPred), 
  vec3 xEst, t1, t2, t3, t4;
  copy_bound_vec3(xEst,xPred,stateBound); // Bound state to avoid overflow.
  float yEst = vec3_mul_inner(Cd,xEst);
  mat3x3_mul_vec3(t1, Ad, xEst); // t1 = Ad*xEst
  vec3_scale(t2,Bd,u); // t2 = Bd*u
  vec3_scale(t3, Ld, yMeas-yEst); // t3 = Ld*(yMeas-yEst)
  vec3_add(t4,t2,t1); // t4 = t2+t1 = Ad*xEst+Bd*u
  vec3_add(xPred, t4,t3); // xPred = Ad*xEst+Bd*u+Ld*(yMeas-yEst)

  // Write motor cmd to H-bridge.  FIX THIS
  // Clip the command u.
  u = min(uMax,u);
  u = max(-uMax,u);
  
  // DO NOT CHANGE SIGNS OF U, BackEMF Will BECOME INCONSISTENT!
  my6302.hbridgeWrite(nomCmdL+u, pwmB1);  
  my6302.hbridgeWrite(nomCmdR-u, pwmB2_T);  

  // Send data to monitor.
  if(loopCounter <  plotIncrement) loopCounter++;
  else {
    loopCounter = 1;
    if(pauseVal == 0) {
      // Use that X[0]=yMeas for prop arm.
      float loopStatus[10];
      loopStatus[PLOT_YM] = yMeas*10;
      loopStatus[PLOT_YE] = yEst*10;
      loopStatus[PLOT_YD] = y_d*10;
      loopStatus[PLOT_M1] = xMeas[1];
      loopStatus[PLOT_E1] = xEst[1];
      loopStatus[PLOT_M2] = 100*xMeas[2];
      loopStatus[PLOT_E2] = 100*xEst[2];
      loopStatus[PLOT_DM] = 10*u;
      //loopStatus[PLOT_INT] = xSum;
      my6302.sendStatus(9,loopStatus);
      headRoom = float(controllerUpdatePeriod);
    }
  }
}


  


  
  


  
