function sys = labFitter(numPoles,numZeros,useSyntheticData)
% Lab 12 MATLAB code for estimating model from frequency sweep data
% and then synthesizing a controller.  Many issues have been clarified
% after lecture!

% Look for ***FIX THIS!!!*** where you need to add/edit

% This makes sure bode plots have continuous phase.
optBode = bodeoptions;
optBode.PhaseWrapping = 'on';
optBode.PhaseWrappingBranch = -360;

% Filename of freq sweep dataset 
if ~useSyntheticData 
    sweepfname = 'sweep.csv';  % ***FIX THIS!!*** Decide File Name
    M = readmatrix(sweepfname);  % Read CSV file and average sweeps together
    M = M/1000.0;
else
    % Create synthetic data in sweeper format ***********************
    % Create log spaced frequency points from 1/4 to 4 hertz
    F = 2.^linspace(-2,2,30)';
    jW = 1i*2*pi*F;
    
    % Frequency Response for a plant stabilized with a PD controller
    % Ydesired to Y output, or closed-loop transfer function*********
    %   HYd2Y(s) = K(s)H(s)/(1 + K(s)H(s))
    %   This case: K(s) = (1.8 + 0.2s),  H(s) = 10/((s+0.1)(s+1)(s+10))
    %   KjW = (1.8 + 0.2*jW), HjW = 10/((jW+0.1)(jW+1)(jW+10)
    %   HYd2YjW = KjW*HjW/(1+KjW*Hjw), which can be  simplified.
    HYd2YjW = 10*(1.8 + 0.2*jW)./((jW+0.1).*(jW+1).*(jW+10) + 10*(1.8+0.2*jW)); 
    
    % Transfer function from Y_desired to U (plant input)
    %   HYd2U(s) = HYd2Y(s)/H(s), or HYd2UjW = HYd2Yjw/Hjw
    HYd2UjW = HYd2YjW.*((jW+10).*(jW+1).*(jW+0.1)/10.0);  % Notice "dot-divide"
   
    % Fill in the five column M array
    M = zeros(length(jW),6);
    M(:,1) = F; % Second column has frequency points
    M(:,2) = abs(HYd2YjW); % output SSS magnitudes
    M(:,3) = (180/pi)*angle(HYd2YjW); % output SSS phases in degrees.
    M(:,4) = abs(HYd2UjW); % controller SSS magnitudes
    M(:,5) = (180/pi)*angle(HYd2UjW); % controller SSS phases
end

% Get the frequency, Magnitude and wrapped phase from the data.
% Note that there are two transfer functions, 
% 1) Closed-loop desired to arm angle, G(s)
% 2) Closed-loop desired to motor command, C(s)
Frequency = M(:,1);
MagG = M(:,2);
PhaseG = M(:,3);
PhaseG(PhaseG>20) = PhaseG(PhaseG>20) - 360;
MagC = M(:,4);
PhaseC = M(:,5);

% If you let the system take n cycles of frequency data, the lines
% below compute the average magnitude and phase over n averages.
freq = sort(unique(Frequency)); % Pull out one copy of each frequency
count = zeros(size(freq)); 
magG = zeros(size(freq));
phaseG = zeros(size(freq));
magC = zeros(size(freq));
phaseC = zeros(size(freq));

freqspot = containers.Map;
for n = 1:size(freq)
    freqspot(num2str(freq(n)))=n;
end

for n = 1:size(Frequency)
    tempn = freqspot(num2str(Frequency(n)));
    count(tempn)=count(tempn)+1;
    magG(tempn)=magG(tempn)+ MagG(n);
    phaseG(tempn)=phaseG(tempn)+PhaseG(n);
    magC(tempn) = magC(tempn)+MagC(n);
    phaseC(tempn) = phaseC(tempn)+PhaseC(n);
end

for n = 1:size(freq)
    magG(n) = magG(n)/count(n);
    phaseG(n) = phaseG(n)/count(n);
    magC(n) = magC(n)/count(n);
    phaseC(n) = phaseC(n)/count(n);
end

% ============
% Convert to complex frequency response and create idfrd object
% ============
y_resp = magG.*exp(phaseG*pi/180*1i);   % Y is the output (angle sensor measurement)
u_resp = magC.*exp(phaseC*pi/180*1i); % U: motor cmd (volts from PWM)
Ymeas = idfrd(y_resp,freq*2*pi,0);    % Ymeas is the TF from Y_D to Y
Umeas = idfrd(u_resp, freq*2*pi, 0);  % Umeas is the TF from Y_D to U

% These frequency responses can be multiplied and divided!
Hmeas = Ymeas/Umeas;   % ***FIX THIS!!** Incorrect formula for plant H.

figure(1); clf;

bodeplot(Ymeas,optBode); hold on;
bodeplot(Umeas,optBode); 
bodeplot(Hmeas,optBode); 
hold off;
legend('Measured Y/Y_D', 'Measured U/Y_D','Measured H');

% ============
% Use TFEST to estimate a rational transfer function
% ============
%optvals = tfestOptions('EnforceStability', true);
%predicted_system = tfest(Hmeas,numPoles,numZeros,optvals);
predicted_system = tfest(Hmeas,numPoles,numZeros);
tf_predicted = tf(predicted_system);

figure(2); clf;
bodeplot(tf_predicted,Hmeas,optBode);
legend('Estimated Rational Y/U', 'Measured Y/U');

% ======================
% Convert CT plant transfer function to DT state space model
% ======================
sys = ss(predicted_system); % Create a CT system
