% This script creates a state-space model, computes gains, and writes to a
% file read when compiling the 6302_SSMS_control file.
%
%close all; % close all the plots.
clearvars;

% Flags for this file
perturb = true; % Perturb Estimated State to time recovery.
doIntegral = false;  % Figure out changes for integrator
useAnalytic = true;  % Switch to false to use the fitter.

if useAnalytic
    sys = propModel(); % Generate the propeller state-space model
else
    numPoles = 0;  % FIX this when using fitter! How many poles, how many zeros.
    numZeros = 0;
    useSyntheticData = false;
    sys = labFitter(numPoles,numZeros,useSyntheticData);
end

E = sys.e;
A = sys.a;
B = sys.b;
C = sys.c;

% Convert to a CT and a DT system
DeltaT = 1.0e-3; % MUST MATCH TEENSY CONTROLLER UPDATE RATE!!!!!
prop_sys_d = c2d(sys, DeltaT);
Ad = prop_sys_d.a;
Bd = prop_sys_d.b;
Cd = prop_sys_d.c; 
stateNames = prop_sys_d.statename;

% ****  WE NOW HAVE BOTH CT (ABCE) and DT (AdBdCd) state-space models

% Find the gains for a measured-state and estimated-state DT controller
if doIntegral
    [Krd,Kd,Ld,Ad,Bd,Cd] = dt2obs(Ad,Bd,Cd,useAnalytic,DeltaT);
else
    [Krd,Kd,Ld,Ad,Bd,Cd] = dt2obs(Ad,Bd,Cd,useAnalytic);
end

% Noise model for the simulator, very approximate based on tests on our 
% setup.  Note angle noise is much lower than omega and backEMF noise.  
if useAnalytic
    NoiseAmps = 0.5*[0.001,1.0,0.05]';  % Lower noise on angle
else
    NoiseAmps = 0.0*[1.0,1.0,1.0]';  % FIX THIS when using fitter
end

% Set up variables for simulator when using integral
if doIntegral
    Bi = [0,0,0,-DeltaT]';
    NoiseAmps(4) = 0.0;
    stateNames{end+1} = 'Sum';
else
    Bi = [0,0,0]';
end

% Simulate the controller
DTSim(Ad,Bd,Cd,Krd,Kd,Ld,DeltaT,Bi,NoiseAmps,perturb,stateNames);

% Output gains and model to a file compiled by Teensyduino
fname = 'obs6302.h'; % If you change this name, change it in sketch!!!
printTeensyObserver(Ad,Bd,Cd,Krd,Kd,Ld,fname)


